@@ -152,36 +152,39 @@ def _app_channel_configuration(ch_info):
 
 
 def _export_gpio(ch_info):
-    if os.path.exists(_SYSFS_ROOT + "/" + ch_info.gpio_name):
-        return
-
-    with open(_SYSFS_ROOT + "/export", "w") as f_export:
-        f_export.write(str(ch_info.gpio))
+    if not os.path.exists(_SYSFS_ROOT + "/" + ch_info.gpio_name):
+        with open(_SYSFS_ROOT + "/export", "w") as f_export:
+            f_export.write(str(ch_info.gpio))
 
     while not os.access(_SYSFS_ROOT + "/" + ch_info.gpio_name + "/value",
                         os.R_OK | os.W_OK):
         time.sleep(0.01)
 
+    ch_info.f_direction = open(_SYSFS_ROOT + "/" + ch_info.gpio_name + "/direction", 'w')
+    ch_info.f_value = open(_SYSFS_ROOT + "/" + ch_info.gpio_name + "/value", 'r+')
+
 
 def _unexport_gpio(ch_info):
-    if not os.path.exists(_SYSFS_ROOT + "/" + ch_info.gpio_name):
-        return
+    ch_info.f_direction.close()
+    ch_info.f_value.close()
 
-    with open(_SYSFS_ROOT + "/unexport", "w") as f_unexport:
-        f_unexport.write(str(ch_info.gpio))
+    if os.path.exists(_SYSFS_ROOT + "/" + ch_info.gpio_name):
+        with open(_SYSFS_ROOT + "/unexport", "w") as f_unexport:
+            f_unexport.write(str(ch_info.gpio))
 
 
 def _output_one(ch_info, value):
-    with open(_SYSFS_ROOT + "/" + ch_info.gpio_name + "/value", 'w') as value_file:
-        value_file.write(str(int(bool(value))))
+    ch_info.f_value.seek(0)
+    ch_info.f_value.write(str(int(bool(value))))
+    ch_info.f_value.flush()
 
 
 def _setup_single_out(ch_info, initial=None):
     _export_gpio(ch_info)
 
-    gpio_dir_path = _SYSFS_ROOT + "/" + ch_info.gpio_name + "/direction"
-    with open(gpio_dir_path, 'w') as direction_file:
-        direction_file.write("out")
+    ch_info.f_direction.seek(0)
+    ch_info.f_direction.write("out")
+    ch_info.f_direction.flush()
 
     if initial is not None:
         _output_one(ch_info, initial)
@@ -192,9 +195,9 @@ def _setup_single_out(ch_info, initial=None):
 def _setup_single_in(ch_info):
     _export_gpio(ch_info)
 
-    gpio_dir_path = _SYSFS_ROOT + "/" + ch_info.gpio_name + "/direction"
-    with open(gpio_dir_path, 'w') as direction:
-        direction.write("in")
+    ch_info.f_direction.seek(0)
+    ch_info.f_direction.write("in")
+    ch_info.f_direction.flush()
 
     _channel_configuration[ch_info.channel] = IN
 
@@ -224,18 +227,19 @@ def _pwm_enable_path(ch_info):
 
 
 def _export_pwm(ch_info):
-    if os.path.exists(_pwm_path(ch_info)):
-        return
-
-    with open(_pwm_export_path(ch_info), 'w') as f:
-        f.write(str(ch_info.pwm_id))
+    if not os.path.exists(_pwm_path(ch_info)):
+        with open(_pwm_export_path(ch_info), 'w') as f:
+            f.write(str(ch_info.pwm_id))
 
     enable_path = _pwm_enable_path(ch_info)
     while not os.access(enable_path, os.R_OK | os.W_OK):
         time.sleep(0.01)
 
+    ch_info.f_duty_cycle = open(_pwm_duty_cycle_path(ch_info), 'r+')
 
 def _unexport_pwm(ch_info):
+    ch_info.f_duty_cycle.close()
+
     with open(_pwm_unexport_path(ch_info), 'w') as f:
         f.write(str(ch_info.pwm_id))
 
@@ -255,13 +259,14 @@ def _set_pwm_duty_cycle(ch_info, duty_cycle_ns):
     # this check only for the 0 duty cycle case, to avoid having to read the
     # current value every time the duty cycle is set.
     if not duty_cycle_ns:
-        with open(_pwm_duty_cycle_path(ch_info), 'r') as f:
-            cur = f.read().strip()
+        ch_info.f_duty_cycle.seek(0)
+        cur = ch_info.f_duty_cycle.read().strip()
         if cur == '0':
             return
 
-    with open(_pwm_duty_cycle_path(ch_info), 'w') as f:
-        f.write(str(duty_cycle_ns))
+    ch_info.f_duty_cycle.seek(0)
+    ch_info.f_duty_cycle.write(str(duty_cycle_ns))
+    ch_info.f_duty_cycle.flush()
 
 
 def _enable_pwm(ch_info):
@@ -331,12 +336,25 @@ def getmode():
     return _gpio_mode
 
 
+# Mutable class to represent a default function argument.
+# See https://stackoverflow.com/a/57628817/2767322
+class _Default:
+    def __init__(self, val):
+        self.val = val
+
+
 # Function used to setup individual pins or lists/tuples of pins as
 # Input or Output. Param channels must an integer or list/tuple of integers,
 # direction must be IN or OUT, pull_up_down must be PUD_OFF, PUD_UP or
 # PUD_DOWN and is only valid when direction in IN, initial must be HIGH or LOW
 # and is only valid when direction is OUT
-def setup(channels, direction, pull_up_down=PUD_OFF, initial=None):
+def setup(channels, direction, pull_up_down=_Default(PUD_OFF), initial=None):
+    if pull_up_down in setup.__defaults__:
+        pull_up_down_explicit = False
+        pull_up_down = pull_up_down.val
+    else:
+        pull_up_down_explicit = True
+
     ch_infos = _channels_to_infos(channels, need_gpio=True)
 
     # check direction is valid
@@ -347,10 +365,12 @@ def setup(channels, direction, pull_up_down=PUD_OFF, initial=None):
     if direction == OUT and pull_up_down != PUD_OFF:
         raise ValueError("pull_up_down parameter is not valid for outputs")
 
-    # check if pullup/down value is valid
+    # check if pullup/down value is specified and/or valid
+    if pull_up_down_explicit:
+        warnings.warn("Jetson.GPIO ignores setup()'s pull_up_down parameter")
     if (pull_up_down != PUD_OFF and pull_up_down != PUD_UP and
             pull_up_down != PUD_DOWN):
-        raise ValueError("Invalid value for pull_up_down - should be either"
+        raise ValueError("Invalid value for pull_up_down; should be one of"
                          "PUD_OFF, PUD_UP or PUD_DOWN")
 
     if _gpio_warnings:
@@ -411,9 +431,9 @@ def input(channel):
     if app_cfg not in [IN, OUT]:
         raise RuntimeError("You must setup() the GPIO channel first")
 
-    with open(_SYSFS_ROOT + "/" + ch_info.gpio_name + "/value") as value:
-        value_read = int(value.read())
-        return value_read
+    ch_info.f_value.seek(0)
+    value_read = int(ch_info.f_value.read())
+    return value_read
 
 
 # Function used to set a value to a channel or list/tuple of channels.
@@ -610,6 +630,8 @@ class PWM(object):
         _export_pwm(self._ch_info)
         self._started = False
         _set_pwm_duty_cycle(self._ch_info, 0)
+        # Anything that doesn't match new frequency_hz
+        self._frequency_hz = -1 * frequency_hz
         self._reconfigure(frequency_hz, 0.0)
 
         _channel_configuration[channel] = HARD_PWM
@@ -641,20 +663,22 @@ class PWM(object):
         if duty_cycle_percent < 0.0 or duty_cycle_percent > 100.0:
             raise ValueError("")
 
-        restart = start or self._started
-        if self._started:
+        freq_change = start or (frequency_hz != self._frequency_hz)
+        stop = self._started and freq_change
+        if stop:
             self._started = False
             _disable_pwm(self._ch_info)
 
-        self._frequency_hz = frequency_hz
-        self._period_ns = int(1000000000.0 / frequency_hz)
-        _set_pwm_period(self._ch_info, self._period_ns)
+        if freq_change:
+            self._frequency_hz = frequency_hz
+            self._period_ns = int(1000000000.0 / frequency_hz)
+            _set_pwm_period(self._ch_info, self._period_ns)
 
         self._duty_cycle_percent = duty_cycle_percent
         self._duty_cycle_ns = int(self._period_ns *
                                   (duty_cycle_percent / 100.0))
         _set_pwm_duty_cycle(self._ch_info, self._duty_cycle_ns)
 
-        if restart:
+        if stop or start:
             _enable_pwm(self._ch_info)
             self._started = True
